#include <avr\io.h>
#define F_CPU 16000000

#define DIROUT DDRC
#define PORTOUT PORTC
#define SENSORS 8

#define ENA 4
#define ENB 5
#define IN1 0
#define IN2 1
#define IN3 2
#define IN4 3

void setM1F()
{
  PORTOUT |= _BV(IN1);
  PORTOUT &= ~_BV(IN2);
}

void setM2F()
{
  PORTOUT |= _BV(IN3);
  PORTOUT &= ~_BV(IN4);
}

void setM1B()
{
  PORTOUT |= _BV(IN2);
  PORTOUT &= ~_BV(IN1);
}

void setM2B()
{
  PORTOUT |= _BV(IN4);
  PORTOUT &= ~_BV(IN3);
}

void fwd()
{
  setM1F();
  setM2F();
  	
  PORTOUT |= _BV(ENA);
  PORTOUT |= _BV(ENB);
}

void left()
{
  setM2F();
	
  PORTOUT &= ~_BV(ENA);
  PORTOUT |= _BV(ENB);
}

void right()
{
  setM1F();
	
  PORTOUT &= ~_BV(ENB);
  PORTOUT |= _BV(ENA);
}

void rleft()
{
  setM1B();  
  setM2F(); 
	
  PORTOUT |= _BV(ENA);
  PORTOUT |= _BV(ENB);
}
	
void rright()
{
  setM2B();  
  setM1F();
	
  PORTOUT |= _BV(ENA);
  PORTOUT |= _BV(ENB);
}

uint8_t sense()
{
  uint8_t i, c = 0, s = 0;

  /* compute sensor average */
  for (i = 0; i < SENSORS; i++) {
    if (PINA & _BV(i)) {
      s += i + 1;
      c++;
    }
  }
	
  return (c == 0) ? 0 : s / c;
}

int main()
{
  DDRA = 0x00;  /* sensor input */
  PORTA = 0xFF; /* pull-up */

  DIROUT = 0xFF;
  PORTOUT = 0x00;
	
  uint8_t ld = 0;
  while (1) {
    uint8_t lp = sense();
    if (lp == 0) {        /* cannot see line */
	if (ld == 0) {
	  rleft();
	} else {
	  rright();
	}
    } else if (lp < 4) {  /* line is to the left */
	left();
	ld = 0;
    } else if (lp > 5) {  /* line is to the right */
	right();
	ld = 1;
    } else {              /* line is centered */
	fwd();
    }
  }
	
  return 0;
}
